<?php

namespace Drupal\sms_netgsm\Plugin\SmsGateway;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\sms\Message\SmsDeliveryReport;
use Drupal\sms\Message\SmsMessageInterface;
use Drupal\sms\Message\SmsMessageReportStatus;
use Drupal\sms\Message\SmsMessageResult;
use Drupal\sms\Plugin\SmsGatewayPluginBase;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Implements Netgsm gateway.
 *
 * @SmsGateway(
 *   id = "netgsm",
 *   label = @Translation("Netgsm"),
 * )
 */
class Netgsm extends SmsGatewayPluginBase implements ContainerFactoryPluginInterface {

  /**
   * Define constants for this class.
   */
  const BAD_REQUEST = 400;
  const UNAUTHORIZED = 401;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * Constructs a new Netgsm plugin.
   *
   * @param array $configuration
   *   The configuration to use and build the sms gateway.
   * @param string $plugin_id
   *   The gateway id.
   * @param mixed $plugin_definition
   *   The gateway plugin definition.
   * @param \GuzzleHttp\ClientInterface $client
   *   The HTTP client.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ClientInterface $client) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('http_client'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'username' => '',
      'password' => '',
      'sender' => 'Netgsm',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $config = $this->getConfiguration();

    $form['netgsm'] = [
      '#type' => 'details',
      '#title' => $this->t('Netgsm'),
      '#open' => TRUE,
    ];
    $form['netgsm']['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#default_value' => $config['username'],
      '#required' => TRUE,
    ];
    $form['netgsm']['password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#default_value' => $config['password'],
      '#required' => TRUE,
    ];
    $form['netgsm']['sender'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Sender'),
      '#default_value' => $config['sender'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Sanitize the credentials.
    $username = Html::escape($form_state->getValue('username'));
    $password = Html::escape($form_state->getValue('password'));
    $sender = Html::escape($form_state->getValue('sender'));

    // Save the credentials.
    $this->configuration['username'] = trim($username);
    $this->configuration['password'] = trim($password);
    $this->configuration['sender'] = trim($sender);
  }

  /**
   * {@inheritdoc}
   */
  public function send(SmsMessageInterface $sms_message) {
    $result = new SmsMessageResult();
    $report = new SmsDeliveryReport();

    // Get the message plain text.
    $message = $sms_message->getMessage();

    // Get the Netgsm statuses.
    $err_conditions = $this->getErrorConditions();

    // Get the Unix timestamp.
    $timestamp = \time();

    foreach ($sms_message->getRecipients() as $recipient) {
      // Prepare the POST method URL.
      $url = Url::fromUri('https://api.netgsm.com.tr/sms/send/get', [
        'query' => [
          'usercode' => $this->configuration['username'],
          'password' => $this->configuration['password'],
          'gsmno' => $recipient,
          'message' => $message,
          'msgheader' => $this->configuration['sender'],
        ],
      ]);

      try {
        $response = $this->client->request('POST', $url->toString());
        $status_code = $response->getBody()->getContents();
        $err_condition = $err_conditions[$status_code] ?? NULL;
        $exception_code = $status_code == '30' ? self::UNAUTHORIZED : self::BAD_REQUEST;

        // Sets SMS message report values.
        $report->setMessageId($sms_message->id());
        $report->setStatusMessage($message);
        $report->setStatusTime($timestamp);
        $report->setRecipient($recipient);

        // Throws an HTTP exception if error conditions
        // match the returned content.
        if ($err_condition) {
          $report->setStatus(SmsMessageReportStatus::ERROR);
          throw new HttpException($exception_code, $err_conditions[$status_code]);
        }

        // Set the report status and show status message.
        $report->setStatus(SmsMessageReportStatus::DELIVERED);
        $this->messenger()->addMessage($this->t('The SMS has been successfully sent to %recipient.', [
          '%recipient' => $recipient,
        ]));
      }
      catch (HttpException $e) {
        $this->messenger()->addError($e->getMessage());
      }
    }

    return $result;
  }

  /**
   * Get the error conditions code.
   *
   * @return array
   *   Return the error conditions code.
   */
  protected function getErrorConditions() {
    return [
      '20' => 'Invalid characters or exceeded the standard maximum number of message characters.',
      '30' => 'Invalid username or password.',
      '40' => 'Invalid sender name.',
      '50' => 'IYS controlled submissions cannot be made with your subscriber account.',
      '51' => 'Indicates that the IYS Brand information defined for your subscription is not found.',
      '70' => 'Invalid query parameters.',
      '80' => 'Sending limit exceeded.',
      '85' => 'Duplicate submission limit exceeded. More than 20 tasks cannot be created for the same number within 1 minute.',
      '100' => 'Netgsm system failure.',
      '101' => 'Netgsm system failure.',
    ];
  }

}
